#!/usr/bin/env python3
"""
rogue8.py

We've got multiple monsters happening at this point, but it would
be nice if we could include weapons or items in the room as well.

Just as we had an array of monsters set up that we could work with,
let's now set up an array of items, and see if we can figure out a 
way for our character to interact with those items.

Before we do that, however, let's remove some of the code from our program. 

@author Richard White
@version 2015-06-07
"""

import os               # so that we can clear the screen
import random
from Location import *        # These .py file must all be in the same directory
from Room import *            # as the main program so that they can be found
from Creature import *         # when we import them. 


def initialize_values():
    """Create the initial states for the room(s), the hero, the monster(s), and later,
    items in the game."""
    
    def random_location(room):
        """We'll use this to establish random locations for monsters."""
        random_x = random.randrange(room.getUpperLeft().getX() + 1, room.getLowerRight().getX())
        random_y = random.randrange(room.getLowerRight().getY() + 1, room.getUpperLeft().getY())
        return Location(random_x, random_y)
    
    # Create a room 
    the_room = Room(Location(5,10),Location(30, 0))
    # Create an adventure3
    hero = Creature("@","the hero",Location(int(the_room.getWidth()/2), \
                                            int(the_room.getHeight()/2)), \
                                            True, 2, 2, 10)
    # Create a monster list
    number_of_monsters = 3
    monster_list = []
    for i in range(number_of_monsters):
        monster_random_location = random_location(the_room)
        while monster_random_location == hero.getPosition():
            monster_random_location = random_location(the_room)
        monster_list.append(Creature("M","a monster",random_location(the_room)))

    return the_room, hero, monster_list

def monster_at(monster_list, a_location):
    for monster in monster_list:
        if monster.getPosition() == a_location:
            return monster
    return None

def display_room(room, hero, monster_list):
    """Takes the room, hero, and monster objects and displays them
    on the screen."""
    os.system('clear')
    
    for row in range(room.getUpperLeft().getY(), room.getLowerRight().getY() - 1, -1):
        for col in range(room.getUpperLeft().getX(), room.getLowerRight().getX() + 1):
            if row == room.getUpperLeft().getY() or row == room.getLowerRight().getY(): 
                print("-",end='')
            elif col == room.getUpperLeft().getX() or col == room.getLowerRight().getX(): 
                print("|",end='')
            elif monster_at(monster_list, Location(col, row)) != None:
                print(monster_at(monster_list, Location(col, row)).getSymbol(),end='')
            elif Location(col, row) == hero.getPosition():
                print(hero.getSymbol(),end='')
            else: 
                print('.',end='')
        print()

def is_legal_position(a_location, a_room):
    if a_location.getX() > a_room.getUpperLeft().getX() and \
       a_location.getX() < a_room.getLowerRight().getX() and \
       a_location.getY() > a_room.getLowerRight().getY() and \
       a_location.getY() < a_room.getUpperLeft().getY():
        return True
    else:
        return False

def process_attack(party1, party2, weapon):
    """This function comes up with a random value that is used in calculating the winner
    of the attack, and takes away points from the loser.
    party1 is the attacker, party2 is the attackee, and weapon is the maximum damage the
    weapon can do"""
    roll = int(random.random() * 20) + 1
    if (roll + party1.getAttack() > 10 + party2.getDefense()):
        print(party1.getName(),"hits",party2.getName())
        damage = int(random.random() * weapon) + 1
        party2.setHitPoints(party2.getHitPoints() - damage)
        # report results
        if party2.getHitPoints() > 0:
            print(party2.getName(),"has",party2.getHitPoints(),"hit points left.")
        else:
            print(party2.getName(),"has died!")
    else:
        print(party1.getName(),"missed!")
    pause = input("[Enter] to continue...")

def process_move(room, hero, monster_list):
    """Get move input from the user. Currently this is just a direction,
    a,s,d,w,q,e,z,c, or the 'Q' character to quit the game. More commands will be
    added later on."""
    move = input("Move: ")
    # Try to move but check location too!
    potential_location = Location(-1,-1)
    if move == "a":
        potential_location = Location(hero.getPosition().getX() - 1, hero.getPosition().getY())
    elif move == "d":
        potential_location = Location(hero.getPosition().getX() + 1, hero.getPosition().getY())
    elif move == "w":
        potential_location = Location(hero.getPosition().getX(), hero.getPosition().getY() + 1)
    elif move == "s":
        potential_location = Location(hero.getPosition().getX(), hero.getPosition().getY() - 1)
    elif move == "q":
        potential_location = Location(hero.getPosition().getX() - 1, hero.getPosition().getY() + 1)
    elif move == "e":
        potential_location = Location(hero.getPosition().getX() + 1, hero.getPosition().getY() + 1)
    elif move == "c":
        potential_location = Location(hero.getPosition().getX() + 1, hero.getPosition().getY() - 1)
    elif move == "z":
        potential_location = Location(hero.getPosition().getX() - 1, hero.getPosition().getY() - 1)
    elif move == "Q": 
        return "Q"

    # Hero moves/attacks
    moving = True
    if is_legal_position(potential_location, room):
        for monster in monster_list:
            if monster_at(monster_list, potential_location) != None and monster.isAlive():
                if potential_location == monster_at(monster_list, potential_location).getPosition():
                    process_attack(hero, monster_at(monster_list, potential_location), 6)
                    moving = False
                    break
        if moving:
            hero.setPosition(potential_location)
    else:
        pause = input("Sorry, illegal move! [Enter] to continue...")

    # Update the room's display on the screen
    display_room(room, hero, monster_list)
    
    # Monster's turn to move/hit
    for monster in monster_list:
        if monster.isAlive():
            potential_location = Location(monster.getPosition().getX(), monster.getPosition().getY())
            if monster.getPosition().getX() > hero.getPosition().getX(): 
                potential_location = Location(monster.getPosition().getX() - 1, potential_location.getY())
            elif monster.getPosition().getX() < hero.getPosition().getX(): 
                potential_location = Location(monster.getPosition().getX() + 1, potential_location.getY())

            if monster.getPosition().getY() > hero.getPosition().getY(): 
                potential_location = Location(potential_location.getX(), monster.getPosition().getY() - 1)
            elif monster.getPosition().getY() < hero.getPosition().getY(): 
                potential_location = Location(potential_location.getX(), monster.getPosition().getY() + 1)
            # Monster moves/attacks
            if is_legal_position(potential_location, room):
                if potential_location == hero.getPosition():  # if hero is where monster wants to move
                    process_attack(monster, hero, 6)
                else:
                    monster.setPosition(potential_location)

    return move


def main():
    the_room, hero, monster_list = initialize_values()
    # Get move/attack commands for the hero
    move = ""
    while move != "Q" and hero.isAlive(): # Main game loop
        display_room(the_room, hero, monster_list)
        move = process_move(the_room, hero, monster_list)

    if move == "Q":
        print("""Thanks for playing our game!""")
    else:
        print("""You died! Better luck next time!""")

if __name__ == "__main__":
    main()


